from http import HTTPStatus
from typing import Any, Optional, Union, cast

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.bot_created_response import BotCreatedResponse
from ...models.http_validation_error import HTTPValidationError
from ...models.new_bot_request import NewBotRequest
from ...types import Response


def _get_kwargs(
    *,
    body: NewBotRequest,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/register-bot",
    }

    _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Any, BotCreatedResponse, HTTPValidationError]]:
    if response.status_code == 200:
        response_200 = BotCreatedResponse.from_dict(response.json())

        return response_200
    if response.status_code == 404:
        response_404 = cast(Any, None)
        return response_404
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Any, BotCreatedResponse, HTTPValidationError]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    body: NewBotRequest,
) -> Response[Union[Any, BotCreatedResponse, HTTPValidationError]]:
    """Register Bot

    Args:
        body (NewBotRequest):  Example: {'api_key': '123e4567-e89b-12d3-a456-426614174000',
            'api_secret': 'qwerty123456', 'bot_token':
            '7516569554:AAFj-G7jpoBuj2UK5wBB7xEQxBuRVAPQCrU', 'integration_uuid':
            '123e4567-e89b-12d3-a456-426614174000'}.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, BotCreatedResponse, HTTPValidationError]]
    """

    kwargs = _get_kwargs(
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Union[AuthenticatedClient, Client],
    body: NewBotRequest,
) -> Optional[Union[Any, BotCreatedResponse, HTTPValidationError]]:
    """Register Bot

    Args:
        body (NewBotRequest):  Example: {'api_key': '123e4567-e89b-12d3-a456-426614174000',
            'api_secret': 'qwerty123456', 'bot_token':
            '7516569554:AAFj-G7jpoBuj2UK5wBB7xEQxBuRVAPQCrU', 'integration_uuid':
            '123e4567-e89b-12d3-a456-426614174000'}.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, BotCreatedResponse, HTTPValidationError]
    """

    return sync_detailed(
        client=client,
        body=body,
    ).parsed


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    body: NewBotRequest,
) -> Response[Union[Any, BotCreatedResponse, HTTPValidationError]]:
    """Register Bot

    Args:
        body (NewBotRequest):  Example: {'api_key': '123e4567-e89b-12d3-a456-426614174000',
            'api_secret': 'qwerty123456', 'bot_token':
            '7516569554:AAFj-G7jpoBuj2UK5wBB7xEQxBuRVAPQCrU', 'integration_uuid':
            '123e4567-e89b-12d3-a456-426614174000'}.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, BotCreatedResponse, HTTPValidationError]]
    """

    kwargs = _get_kwargs(
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
    body: NewBotRequest,
) -> Optional[Union[Any, BotCreatedResponse, HTTPValidationError]]:
    """Register Bot

    Args:
        body (NewBotRequest):  Example: {'api_key': '123e4567-e89b-12d3-a456-426614174000',
            'api_secret': 'qwerty123456', 'bot_token':
            '7516569554:AAFj-G7jpoBuj2UK5wBB7xEQxBuRVAPQCrU', 'integration_uuid':
            '123e4567-e89b-12d3-a456-426614174000'}.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, BotCreatedResponse, HTTPValidationError]
    """

    return (
        await asyncio_detailed(
            client=client,
            body=body,
        )
    ).parsed
